'******************************************************************************
'*  trs80_video.spin - Output a 12 scanline bitmap
'*
'*  This is based on ideas and code from Debug_1pinTV by "Cluso99"
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: yaz80_demo.spin,v 1.8 2010-03-21 12:43:11 pm Exp $
'******************************************************************************
CON

#ifdef NTSC
'' NTSC 64x16
ocols			=	64*6/8			' number of columns
orows			=	16*12/8			' number of rows
ocolsfrq		=	22_478			' Hz per active pixel
oequal1			=	6			' number of equalization pulses
oserr			=	6			' number of serration pulses
oequal2			=	6			' number of equalization pulses
oblank1			=	36			' number of blank lines
oactive			=	16*12			' number of active lines
oblank2			=	25			' number of blank lines
otsync			=	39			' temp sync pulse
ohalf			=	4370			' vscl half line
obackp			=	4171	 		' vscl back porch (sync to active)
ofrontp			=	4145			' vscl front porch (active to sync)
oequal			=	4116			' vscl equalization pulse (sync/2)
osync			=	4136			' vscl sync pulse
osynch			=	4330			' vscl half-sync
oequalh			=	4350			' vscl half-equal
ovsclch			=	4104			' vscl character
ictra			=	12			' CTRA with PLLdiv
ifrqa			=	231701456		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	231701456		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	257446064		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	247148224		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	237642512		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	228840944		' required FRQA value (13.5MHz*8)
#endif

'--------------------------------------------------------------------------------------------------
#ifdef PAL
'' PAL	64x16 at 6x12px == 48x24 at 8x8px
ocols			=	64*6/8			' number of columns
orows			=	16*12/8			' number of rows
ocolsfrq		=	22_321			' Hz per active pixel
oequal1			=	4			' number of equalization pulses
oserr			=	5			' number of serration pulses
oequal2			=	5			' number of equalization pulses
oblank1			=	63			' number of blank lines
oactive			=	192			' number of active lines
oblank2			=	49			' number of blank lines
otsync			=	39			' temp sync pulse 
ohalf			=	4369			' vscl half line
obackp			=	4176			' vscl back porch (sync to active)
ofrontp			=	4139			' vscl front porch (active to sync)
oequal			=	4115			' vscl equalization pulse (sync/2)
osync			=	4135			' vscl sync pulse
osynch			=	4330			' vscl half-sync
oequalh			=	4350			' vscl half-equal
ovsclch			=	4104			' vscl character
ictra			=	12			' CTRA with PLLdiv
ifrqa			=	230083104		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	230083104		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	191735920		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	184066480		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	176987008		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	170431936		' required FRQA value (13.5MHz*8)
#endif
'--------------------------------------------------------------------------------------------------

VAR
	long	cog

PUB start(params) : okay
' bitmap is the address of the bitmap buffer * 32 + the tvPin number

	stop
	Setforxtalfreq					'set ifrqa in hub for the current clkfreq before cognew 
	okay := cog := COGNEW(@entry, params) + 1

PUB stop
	if cog
		COGSTOP(cog - 1)
		cog := 0

PRI Setforxtalfreq
'' Set the frqa value depending on the actual clkfreq
	Case clkfreq
		 80_000_000: xfrqa := ifrqa_80MHz	' 5.00 MHz
		 96_000_000: xfrqa := ifrqa_96MHz	' 6.00 MHz
		100_000_000: xfrqa := ifrqa_100MHz	' 6.25 MHz
		104_000_000: xfrqa := ifrqa_104MHz	' 6.50 MHz
		108_000_000: xfrqa := ifrqa_108MHz	'13.50 MHz

DAT
		org	0
entry
		mov	t1, PAR				' pointer to 4 longs (sync, screen, bitmap, tvPin)
		rdlong	sync_ptr, t1
		add	t1, #8				' skip screen, point to bitmaps
		rdlong	bitmap_ptr, t1
		add	t1, #4				' point to pin number
		rdlong	pin_no, t1			' get TV pin#
		add	bitmap_end, bitmap_ptr		' end of the 2 bitmap buffers

		mov	FRQB, xfrqb			' default = 40/140

		movi	CTRA, #ictra
		mov	FRQA, xfrqa

		movs	CTRB, pin_no			' set pin
		mov	count, #1
		shl	count, pin_no
		mov	DIRA, count			' set pin mask
		mov	count, pin_no			' <=== can save this by next shr #4
		shr	count, #3
		movd	VCFG, count			' set VGroup
		mov	count, #1
		and	pin_no, #7			' <=== poss to save?
		shl	count, pin_no
		movs	VCFG, count			' set VPins
		movi	VCFG, #%0_01_0_0_0_000		' VGA mode, 1 bit per pixel

'**************************************************************************************************
frame		add	framectr, #1			' inc frame counter
		mov	rows, #oequal1
		call	#equalizing			' equalisation pulses
		mov	scanrow, #0
		wrlong	scanrow, sync_ptr
'--------------------------------------------------------------------------------------------------
		mov	rows, #oserr			' serration pulses
:loop		mov	vscl, xsynch			' serration pulse (long)
		waitvid xFF00, #0		
		movi	CTRB, #0			' turn off blank
		mov	vscl, xsync			' serration pulse (short)
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #:loop
'--------------------------------------------------------------------------------------------------
		mov	rows, #oequal2
		call	#equalizing			' equalisation pulses
'==================================================================================================
		mov	rows, #oblank1
		call	#doblank			' blank lines (top)
'==================================================================================================
'display active lines
		mov	bmapptr, bitmap_ptr		' initialize bitmap pointer
		mov	rows, #oactive
doactive	mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	count, #ocols			' characters per line
		mov	vscl, xbackp			' back porch
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	vscl, xvsclch			' 1 PLLA per pixel, 8 PLLA per frame
'--------------------------------------------------------------------------------------------------
'display entire pixel line from bmapptr..
:active
		rdbyte	bits, bmapptr			' read character from HUB RAM
		add	bmapptr, #1
		waitvid	xFF00, bits			' output to screen
		djnz	count, #:active			' do entire line
'--------------------------------------------------------------------------------------------------
		add	fontrow, #1
		cmp	fontrow, #10		WZ	' reached scanline #10 of the bitmap?
	if_z	add	scanrow, #1
	if_z	wrlong	scanrow, sync_ptr		' signal the render cog: do next bitmap line
		cmp	bmapptr, bitmap_end	WZ	' need start over at the first bitmap line?
	if_z	mov	bmapptr, bitmap_ptr
	if_z	mov	fontrow, #0
		mov	vscl, xfrontp			' front porch
		waitvid xFF00, #0			' output blank
		djnz	rows, #doactive			' next row
'==================================================================================================
		mov	rows, #oblank2
		call	#doblank			' blank lines (bottom)
'==================================================================================================
		jmp	#frame
'**************************************************************************************************

equalizing	mov	vscl, xequal			' equalizing pulse (short)
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xequalh			' equalizing pulse (long)
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #equalizing
equalizing_ret
		ret
'--------------------------------------------------------------------------------------------------
doblank
		mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xsynch			' blank line
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	vscl, xhalf			' half line ???
		waitvid xFF00, #0
		djnz	rows, #doblank
doblank_ret
		ret
'**************************************************************************************************

'--------------------------------------------------------------------------------------------------
'constants > $1FF
xhalf		long	ohalf
xbackp		long	obackp
xfrontp		long	ofrontp
xequal		long	oequal
xsync		long	osync
xsynch		long	osynch
xequalh		long	oequalh
xvsclch		long	ovsclch
xfrqa		long	ifrqa
xfrqb		long	ifrqb

xFF00		long	$FF00			' white / black
bitmap_ptr	long	0-0
bitmap_end	long	12*64*6/8
sync_ptr	long	0-0
pin_no		long	0-0

bits		long	0			' current character / character bitmap
bmapptr		long	0			' pointer to bitmap data
framectr	long	0
count		long	0			' all purpose counter
rows		long	0			' row counter
scanrow		long	0			' active character row
fontrow		long	0			' font row counter
t1		long	0

{{
+------------------------------------------------------------------------------------------------------------------------------+
|				    TERMS OF USE: Parallax Object Exchange License					       |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | |files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE	       |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
